@@ -34,13 +34,29 @@
 
 struct xml_writer_private {
 	FILE *fh;
+	ssize_t depth;
 	xmlTextWriterPtr xw;
 	xmlDocPtr doc;
 };
 
+void xml_new_writer(struct xml_writer_private *priv)
+{
+	priv->xw = xmlNewTextWriterDoc(&(priv->doc), 0);
+	if (!priv->xw)
+		fatalx("lldpctl", "cannot create xml writer");
+
+	xmlTextWriterSetIndent(priv->xw, 4);
+
+	if (xmlTextWriterStartDocument(priv->xw, NULL, "UTF-8", NULL) < 0 )
+		fatalx("lldpctl", "cannot start xml document");
+}
+
 void xml_start(struct writer *w , const char *tag, const char *descr ) {
 	struct xml_writer_private *p = w->priv;
 
+	if (p->depth == 0)
+		xml_new_writer(p);
+
 	if (xmlTextWriterStartElement(p->xw, BAD_CAST tag) < 0)
 		log_warnx("lldpctl", "cannot start '%s' element", tag);
 
@@ -48,6 +64,8 @@ void xml_start(struct writer *w , const char *tag, const char *descr ) {
 		if (xmlTextWriterWriteFormatAttribute(p->xw, BAD_CAST "label", "%s", descr) < 0)
 			log_warnx("lldpctl", "cannot add attribute 'label' to element %s", tag);
 	}
+
+	p->depth++;
 }
 
 void xml_attr(struct writer *w, const char *tag, const char *descr, const char *value ) {
@@ -68,27 +86,30 @@ void xml_end(struct writer *w) {
 
 	if (xmlTextWriterEndElement(p->xw) < 0 )
 		log_warnx("lldpctl", "cannot end element");
-}
 
-#define MY_ENCODING "UTF-8"
+	if (--p->depth == 0) {
+		int failed = 0;
+
+		if (xmlTextWriterEndDocument(p->xw) < 0 ) {
+			log_warnx("lldpctl", "cannot finish document");
+			failed = 1;
+		}
+
+		xmlFreeTextWriter(p->xw);
+		if (!failed)
+			xmlDocDump(p->fh, p->doc);
+		xmlFreeDoc(p->doc);
+	}
+}
 
 void xml_finish(struct writer *w) {
 	struct xml_writer_private *p = w->priv;
-	int failed = 0;
-
-	if (xmlTextWriterEndDocument(p->xw) < 0 ) {
-		log_warnx("lldpctl", "cannot finish document");
-		failed = 1;
+	if (p->depth != 0) {
+		log_warnx("lldpctl", "unbalanced tags");
+		/* memory leak... */
 	}
 
-	xmlFreeTextWriter(p->xw);
-
-	if (!failed)
-		xmlDocDump(p->fh, p->doc);
-
-	xmlFreeDoc(p->doc);
-
-	free(w->priv);
+	free(p);
 	free(w);
 }
 
@@ -98,30 +119,16 @@ struct writer *xml_init(FILE *fh) {
 	struct xml_writer_private *priv;
 
 	priv = malloc(sizeof(*priv));
-	priv->fh = fh;
 	if (!priv) {
 		fatalx("lldpctl", "out of memory");
 		return NULL;
 	}
-
-	priv->xw = xmlNewTextWriterDoc(&(priv->doc), 0);
-	if (!priv->xw) {
-		fatalx("lldpctl", "cannot create xml writer");
-		return NULL;
-	}
-
-	xmlTextWriterSetIndent(priv->xw, 4);
-
-	if (xmlTextWriterStartDocument(priv->xw, NULL, MY_ENCODING, NULL) < 0 ) {
-		fatalx("lldpctl", "cannot start xml document");
-		return NULL;
-	}
+	priv->fh = fh;
+	priv->depth = 0;
 
 	result = malloc(sizeof(struct writer));
-	if (!result) {
+	if (!result)
 		fatalx("lldpctl", "out of memory");
-		return NULL;
-	}
 
 	result->priv  = priv;
 	result->start = xml_start;
