@@ -188,7 +188,7 @@ iface_append_vlan(struct lldpd *cfg,
     struct interfaces_device *lower)
 {
 	struct lldpd_hardware *hardware =
-	    lldpd_get_hardware(cfg, lower->name, lower->index, NULL);
+	    lldpd_get_hardware(cfg, lower->name, lower->index);
 	struct lldpd_port *port;
 	struct lldpd_vlan *v;
 
@@ -223,6 +223,7 @@ iface_append_vlan(struct lldpd *cfg,
  *
  * @param vlan  The VLAN interface (used to get VLAN ID).
  * @param upper The upper interface we are currently examining.
+ * @param depth Depth of the stack (avoid infinite recursion)
  *
  * Initially, upper == vlan. This function will be called recursively.
  */
@@ -230,8 +231,16 @@ static void
 iface_append_vlan_to_lower(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *vlan,
-    struct interfaces_device *upper)
+    struct interfaces_device *upper,
+    int depth)
 {
+	if (depth > 5) {
+		log_warn("interfaces",
+		    "BUG: maximum depth reached when applying VLAN %s (loop?)",
+		    vlan->name);
+		return;
+	}
+	depth++;
 	struct interfaces_device *lower;
 	log_debug("interfaces",
 	    "looking to apply VLAN %s to physical interface behind %s",
@@ -243,7 +252,8 @@ iface_append_vlan_to_lower(struct lldpd *cfg,
 		    vlan->name, upper->name);
 		iface_append_vlan_to_lower(cfg,
 		    interfaces, vlan,
-		    upper->lower);
+		    upper->lower,
+		    depth);
 		return;
 	}
 
@@ -262,7 +272,7 @@ iface_append_vlan_to_lower(struct lldpd *cfg,
 		log_debug("interfaces", "VLAN %s on lower interface %s",
 		    vlan->name, upper->name);
 		iface_append_vlan_to_lower(cfg,
-		    interfaces, vlan, lower);
+		    interfaces, vlan, lower, depth);
 	}
 }
 
@@ -283,7 +293,7 @@ interfaces_helper_vlan(struct lldpd *cfg,
 		log_debug("interfaces", "search physical interface for VLAN interface %s",
 		    iface->name);
 		iface_append_vlan_to_lower(cfg, interfaces,
-		    iface, iface);
+		    iface, iface, 0);
 	}
 }
 #endif
@@ -321,8 +331,7 @@ interfaces_helper_chassis(struct lldpd *cfg,
 
 		if ((hardware = lldpd_get_hardware(cfg,
 			    iface->name,
-			    iface->index,
-			    NULL)) == NULL)
+			    iface->index)) == NULL)
 			/* That's odd. Let's skip. */
 			continue;
 
@@ -367,8 +376,8 @@ interfaces_helper_mgmt_for_af(struct lldpd *cfg,
 	struct lldpd_mgmt *mgmt;
 	char addrstrbuf[INET6_ADDRSTRLEN];
 	int found = 0;
-	void *sin_addr_ptr;
-	size_t sin_addr_size;
+	union lldpd_address in_addr;
+	size_t in_addr_size;
 
 	TAILQ_FOREACH(addr, addrs, next) {
 		if (addr->address.ss_family != lldpd_af(af))
@@ -376,24 +385,26 @@ interfaces_helper_mgmt_for_af(struct lldpd *cfg,
 
 		switch (af) {
 		case LLDPD_AF_IPV4:
-			sin_addr_ptr = &((struct sockaddr_in *)&addr->address)->sin_addr;
-			sin_addr_size = sizeof(struct in_addr);
+			in_addr_size = sizeof(struct in_addr);
+			memcpy(&in_addr, &((struct sockaddr_in *)&addr->address)->sin_addr,
+			    in_addr_size);
 			if (global) {
-				if (!IN_IS_ADDR_GLOBAL((struct in_addr *)sin_addr_ptr))
+				if (!IN_IS_ADDR_GLOBAL(&in_addr.inet))
 					continue;
 			} else {
-				if (!IN_IS_ADDR_LINKLOCAL((struct in_addr *)sin_addr_ptr))
+				if (!IN_IS_ADDR_LINKLOCAL(&in_addr.inet))
 					continue;
 			}
 			break;
 		case LLDPD_AF_IPV6:
-			sin_addr_ptr = &((struct sockaddr_in6 *)&addr->address)->sin6_addr;
-			sin_addr_size = sizeof(struct in6_addr);
+			in_addr_size = sizeof(struct in6_addr);
+			memcpy(&in_addr, &((struct sockaddr_in6 *)&addr->address)->sin6_addr,
+			    in_addr_size);
 			if (global) {
-				if (!IN6_IS_ADDR_GLOBAL((struct in6_addr *)sin_addr_ptr))
+				if (!IN6_IS_ADDR_GLOBAL(&in_addr.inet6))
 					continue;
 			} else {
-				if (!IN6_IS_ADDR_LINKLOCAL((struct in6_addr *)sin_addr_ptr))
+				if (!IN6_IS_ADDR_LINKLOCAL(&in_addr.inet6))
 					continue;
 			}
 			break;
@@ -401,14 +412,14 @@ interfaces_helper_mgmt_for_af(struct lldpd *cfg,
 			assert(0);
 			continue;
 		}
-		if (inet_ntop(lldpd_af(af), sin_addr_ptr,
+		if (inet_ntop(lldpd_af(af), &in_addr,
 			addrstrbuf, sizeof(addrstrbuf)) == NULL) {
 			log_warn("interfaces", "unable to convert IP address to a string");
 			continue;
 		}
 		if (cfg->g_config.c_mgmt_pattern == NULL ||
 		    pattern_match(addrstrbuf, cfg->g_config.c_mgmt_pattern, allnegative)) {
-			mgmt = lldpd_alloc_mgmt(af, sin_addr_ptr, sin_addr_size,
+			mgmt = lldpd_alloc_mgmt(af, &in_addr, in_addr_size,
 			    addr->index);
 			if (mgmt == NULL) {
 				assert(errno == ENOMEM); /* anything else is a bug */
@@ -499,43 +510,48 @@ interfaces_helper_port_name_desc(struct lldpd *cfg,
 {
 	struct lldpd_port *port = &hardware->h_lport;
 
-	if (port->p_id_subtype == LLDP_PORTID_SUBTYPE_LOCAL)
-		return;
-
 	/* We need to set the portid to what the client configured.
 	   This can be done from the CLI.
 	*/
-	switch (cfg->g_config.c_lldp_portid_type) {
-	case LLDP_PORTID_SUBTYPE_IFNAME:
-		log_debug("interfaces", "use ifname for %s",
-			  hardware->h_ifname);
-		port->p_id_subtype = LLDP_PORTID_SUBTYPE_IFNAME;
-		port->p_id_len = strlen(hardware->h_ifname);
-		free(port->p_id);
-		if ((port->p_id = calloc(1, port->p_id_len)) == NULL)
-			fatal("interfaces", NULL);
-		memcpy(port->p_id, hardware->h_ifname, port->p_id_len);
-		break;
-	case LLDP_PORTID_SUBTYPE_LLADDR:
-		/* fall through so we use the mac address */
-	default:
-		log_debug("interfaces", "use MAC address for %s",
-			  hardware->h_ifname);
-		port->p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
-		free(port->p_id);
-		if ((port->p_id = calloc(1, ETHER_ADDR_LEN)) == NULL)
-			fatal("interfaces", NULL);
-		memcpy(port->p_id, hardware->h_lladdr, ETHER_ADDR_LEN);
-		port->p_id_len = ETHER_ADDR_LEN;
-	}
+	int has_alias = (iface->alias != NULL && strlen(iface->alias) != 0);
+	int portid_type = cfg->g_config.c_lldp_portid_type;
+	if (portid_type == LLDP_PORTID_SUBTYPE_IFNAME ||
+	    (portid_type == LLDP_PORTID_SUBTYPE_UNKNOWN && has_alias) ||
+	    (portid_type == LLDP_PORTID_SUBTYPE_LOCAL && has_alias)) {
+		if (portid_type != LLDP_PORTID_SUBTYPE_LOCAL) {
+			log_debug("interfaces", "use ifname for %s",
+			    hardware->h_ifname);
+			port->p_id_subtype = LLDP_PORTID_SUBTYPE_IFNAME;
+			port->p_id_len = strlen(hardware->h_ifname);
+			free(port->p_id);
+			if ((port->p_id = calloc(1, port->p_id_len)) == NULL)
+				fatal("interfaces", NULL);
+			memcpy(port->p_id, hardware->h_ifname, port->p_id_len);
+		}
 
-	if (iface->alias != NULL && strlen(iface->alias) != 0) {
 		/* use the actual alias in the port description */
 		log_debug("interfaces", "using alias in description for %s",
 			  hardware->h_ifname);
 		free(port->p_descr);
-		port->p_descr = strdup(iface->alias);
+		if (has_alias) {
+			port->p_descr = strdup(iface->alias);
+		} else {
+			/* We don't have anything else to put here and for CDP
+			 * with need something non-NULL */
+			port->p_descr = strdup(hardware->h_ifname);
+		}
 	} else {
+		if (portid_type != LLDP_PORTID_SUBTYPE_LOCAL) {
+			log_debug("interfaces", "use MAC address for %s",
+			    hardware->h_ifname);
+			port->p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
+			free(port->p_id);
+			if ((port->p_id = calloc(1, ETHER_ADDR_LEN)) == NULL)
+				fatal("interfaces", NULL);
+			memcpy(port->p_id, hardware->h_lladdr, ETHER_ADDR_LEN);
+			port->p_id_len = ETHER_ADDR_LEN;
+		}
+
 		/* use the ifname in the port description until alias is set */
 		log_debug("interfaces", "using ifname in description for %s",
 			  hardware->h_ifname);
@@ -560,6 +576,7 @@ interfaces_helper_physical(struct lldpd *cfg,
 {
 	struct interfaces_device *iface;
 	struct lldpd_hardware *hardware;
+	int created;
 
 	TAILQ_FOREACH(iface, interfaces, next) {
 		if (!(iface->type & IFACE_PHYSICAL_T)) continue;
@@ -567,10 +584,10 @@ interfaces_helper_physical(struct lldpd *cfg,
 
 		log_debug("interfaces", "%s is an acceptable ethernet device",
 		    iface->name);
+		created = 0;
 		if ((hardware = lldpd_get_hardware(cfg,
 			    iface->name,
-			    iface->index,
-			    ops)) == NULL) {
+			    iface->index)) == NULL) {
 			if  ((hardware = lldpd_alloc_hardware(cfg,
 				    iface->name,
 				    iface->index)) == NULL) {
@@ -578,6 +595,21 @@ interfaces_helper_physical(struct lldpd *cfg,
 				    iface->name);
 				continue;
 			}
+			created = 1;
+		}
+		if (hardware->h_flags)
+			continue;
+		if (hardware->h_ops != ops) {
+			if (!created) {
+				log_debug("interfaces",
+				    "interface %s is converted from another type of interface",
+				    hardware->h_ifname);
+				if (hardware->h_ops && hardware->h_ops->cleanup) {
+					hardware->h_ops->cleanup(cfg, hardware);
+					levent_hardware_release(hardware);
+					levent_hardware_init(hardware);
+				}
+			}
 			if (init(cfg, hardware) != 0) {
 				log_warnx("interfaces",
 				    "unable to initialize %s",
@@ -588,11 +620,11 @@ interfaces_helper_physical(struct lldpd *cfg,
 			hardware->h_ops = ops;
 			hardware->h_mangle = (iface->upper &&
 			    iface->upper->type & IFACE_BOND_T);
-			interfaces_helper_add_hardware(cfg, hardware);
-		} else {
-			if (hardware->h_flags) continue; /* Already seen this time */
-			lldpd_port_cleanup(&hardware->h_lport, 0);
 		}
+		if (created)
+			interfaces_helper_add_hardware(cfg, hardware);
+		else
+			lldpd_port_cleanup(&hardware->h_lport, 0);
 
 		hardware->h_flags = iface->flags;   /* Should be non-zero */
 		iface->ignore = 1;		    /* Future handlers
@@ -612,6 +644,8 @@ interfaces_helper_physical(struct lldpd *cfg,
 #ifdef ENABLE_DOT3
 		if (iface->upper && iface->upper->type & IFACE_BOND_T)
 			hardware->h_lport.p_aggregid = iface->upper->index;
+		else
+			hardware->h_lport.p_aggregid = 0;
 #endif
 	}
 }
