@@ -148,14 +148,15 @@ lldpd_af(int af)
 }
 
 #define LLDPD_MGMT_MAXADDRSIZE	16 /* sizeof(struct in6_addr) */
+union lldpd_address {
+	struct in_addr		inet;
+	struct in6_addr		inet6;
+	u_int8_t		octets[LLDPD_MGMT_MAXADDRSIZE]; /* network byte order! */
+};
 struct lldpd_mgmt {
 	TAILQ_ENTRY(lldpd_mgmt) m_entries;
-	int				m_family;
-	union {
-		struct in_addr		inet;
-		struct in6_addr		inet6;
-		u_int8_t		octets[LLDPD_MGMT_MAXADDRSIZE]; /* network byte order! */
-	} m_addr;
+	int			m_family;
+	union lldpd_address	m_addr;
 	size_t 			m_addrsize;
 	u_int32_t		m_iface;
 };
@@ -215,6 +216,11 @@ MARSHAL_STR(lldpd_chassis, c_med_asset)
 MARSHAL_END(lldpd_chassis);
 
 #ifdef ENABLE_CUSTOM
+
+#define CUSTOM_TLV_ADD 		1
+#define CUSTOM_TLV_REPLACE	2
+#define CUSTOM_TLV_REMOVE	3
+
 /* Custom TLV struct as defined on page 35 of IEEE 802.1AB-2005 */
 struct lldpd_custom {
 	TAILQ_ENTRY(lldpd_custom)	next;	/* Pointer to next custom TLV */
@@ -328,6 +334,7 @@ struct lldpd_port_set {
 #ifdef ENABLE_CUSTOM
 	struct lldpd_custom     *custom;
 	int custom_list_clear;
+	int custom_tlv_op;
 #endif
 };
 MARSHAL_BEGIN(lldpd_port_set)
@@ -531,6 +538,8 @@ void	 lldpd_vlan_cleanup(struct lldpd_port *);
 void	 lldpd_pi_cleanup(struct lldpd_port *);
 #endif
 #ifdef ENABLE_CUSTOM
+void     lldpd_custom_tlv_cleanup(struct lldpd_port *, struct lldpd_custom *);
+void     lldpd_custom_tlv_add(struct lldpd_port *, struct lldpd_custom *);
 void     lldpd_custom_list_cleanup(struct lldpd_port *);
 #endif
 
